package cloning;

/* See what happens when you remove "implements Cloneable" */
/* and try to run the example again */

public class Computer implements Cloneable {
    private String brand;
    private Mouse mouse;

    public Computer(String brand, Mouse mouse) {
        this.brand = brand;
        this.mouse = mouse;
    }

    public Mouse getMouse() {
        return mouse;
    }

    /* In this version, the Computer object contains a mutable Mouse.
     * - A shallow copy would copy the reference to the same Mouse object.
     * - We need to **manually clone the Mouse** to ensure each Computer has its own copy.
     */
    @Override
    public Computer clone() {
        try {
            Computer obj = (Computer) super.clone(); // Shallow copy of Computer
            if (mouse != null) {
                obj.mouse = mouse.clone(); // Deep copy the Mouse
            }
            return obj;
        } catch (CloneNotSupportedException e) {
            throw new AssertionError("Cloning not supported, but Cloneable is implemented!", e);
        }
    }

    @Override
    public String toString() {
        return "Brand: " + brand + ", Mouse: " + mouse;
    }

    public static void main(String[] args) {
        Computer s1 = new Computer("Dell", new Mouse("Optical"));
        Computer s2 = s1.clone(); // Deep copy, s2 has its own Mouse

        s2.getMouse().moveMouse(10, 20); // Modify s2's mouse position

        System.out.println("Original: " + s1); // s1's mouse should remain unchanged
        System.out.println("Cloned: " + s2);   // s2's mouse should reflect the movement
    }
}
