package cloning;

/* Notice it is not implementing Cloneable because this one will not 
 * call Object's clone, but the overridden version in the Computer parent class */
public class SuperComputer extends Computer {
    private StringBuffer users;
    private int coolingFactor;
    
    public SuperComputer(String brand, Mouse mouse, int coolingFactor) {
        super(brand, mouse);
        this.coolingFactor = coolingFactor;
        users = new StringBuffer();
    }
    
    public int getCoolingFactor() {
        return coolingFactor;
    }
    
    public void addUser(String name) {
        users.append(name);
    }
    
    public void printUsers() {
        System.out.println("Users: " + users);
    }
    
    /* Since the Computer class's clone() returns a shallow copy (i.e. one shared stringBuffer),
     * we need to manually clone the mutable StringBuffer field
     * so that each SuperComputer has its own independent copy.
     */
    @Override
    public SuperComputer clone() {
        SuperComputer obj = (SuperComputer) super.clone();
        obj.users = new StringBuffer(users); // Create an independent copy of users
        return obj;
    }
    
    public static void main(String[] args) {
        Mouse mouse = new Mouse("Optical");
        SuperComputer superComputer = new SuperComputer("ibm", mouse, 10);
        SuperComputer copy;
        
        superComputer.addUser("John");
        copy = superComputer.clone();
        copy.addUser("Mary");
        
        superComputer.printUsers(); // Expected: Users: John
        copy.printUsers();          // Expected: Users: JohnMary
    }
}

